; conversion factor for eV to 1/um [=1/hc]
(define eV-um-scale (/ 1.23984193))

(define Pt-plasma-frq (* 9.59 eV-um-scale))

(define Pt-f0 0.333)
(define Pt-frq0 1e-10)
(define Pt-gam0 (* 0.080 eV-um-scale))
(define Pt-sig0 (/ (* Pt-f0 (sqr Pt-plasma-frq)) (sqr Pt-frq0)))

(define Pt-f1 0.191)
(define Pt-frq1 (* 0.780 eV-um-scale)) ; 1.590 um
(define Pt-gam1 (* 0.517 eV-um-scale))
(define Pt-sig1 (/ (* Pt-f1 (sqr Pt-plasma-frq)) (sqr Pt-frq1)))

(define Pt-f2 0.659)
(define Pt-frq2 (* 1.314 eV-um-scale)) ; 0.944 um
(define Pt-gam2 (* 1.838 eV-um-scale))
(define Pt-sig2 (/ (* Pt-f2 (sqr Pt-plasma-frq)) (sqr Pt-frq2)))

(define Pt (make medium (epsilon 1.0)
  (E-susceptibilities
     (make drude-susceptibility
       (frequency Pt-frq0) (gamma Pt-gam0) (sigma Pt-sig0))
     (make lorentzian-susceptibility
       (frequency Pt-frq1) (gamma Pt-gam1) (sigma Pt-sig1))
     (make lorentzian-susceptibility
       (frequency Pt-frq2) (gamma Pt-gam2) (sigma Pt-sig2)))))

(set-param! eps-averaging? true)

(set-param! resolution 40)        ; pixels/um

(define-param a 3.5)              ; lattice periodicity
(define-param r 1.1)              ; metal rod radius
(define-param h 0.1)              ; metal rod height
(define-param tmet 1.0)           ; metal layer thickness
(define-param tsub 2.0)           ; substrate thickness
(define-param tabs 5.0)           ; absorber thickness
(define-param tair 4.0)           ; air thickness

(define sz (+ tabs tair h tmet tsub tabs))
(set! geometry-lattice (make lattice (size a a sz)))
(set! pml-layers (list (make pml (thickness tabs) (direction Z) (side High))
		       (make absorber (thickness tabs) (direction Z) (side Low))))

(define lmin 2.0)                 ; source min wavelength
(define lmax 5.0)                 ; source max wavelength
(define fmin (/ lmax))            ; source min frequency
(define fmax (/ lmin))            ; source max frequency
(define fcen (* 0.5 (+ fmin fmax)))
(define df (- fmax fmin))

(define Si (make medium (index 3.5)))

(define-param empty? true)
(if (not empty?)
    (set! geometry (list
		    (make cylinder (material Pt) (radius r) (height h) (center 0 0 (- (* 0.5 sz) tabs tair (* 0.5 h))))
		    (make block (material Pt) (size infinity infinity tmet) (center 0 0 (- (* 0.5 sz) tabs tair h (* 0.5 tmet))))
		    (make block (material Si) (size infinity infinity (+ tsub tabs)) (center 0 0 (- (* 0.5 sz) tabs tair h tmet (* 0.5 (+ tsub tabs))))))))

; rotation angle (in degrees) of source CCW around y axis; 0 degrees along -z axis
(define-param theta 0)
(set! theta (deg->rad theta))

; k with correct length (plane of incidence: XZ)
(define k (vector3-scale fcen (vector3 (sin theta) 0 (cos theta))))

(set! k-point k) ; PBC in XY plane (in units of 2\pi)

(define (pw-amp k x0) (lambda (x) (exp (* 0+1i 2 pi (vector3-dot k (vector3+ x x0))))))

(define src-pos (- (* 0.5 sz) tabs (* 0.2 tair)))
(set! sources (list (make source
		      (src (make gaussian-src (frequency fcen) (fwidth df)))
		      (component Ey) (center 0 0 src-pos) (size a a 0)
		      (amp-func (pw-amp k (vector3 0 0 src-pos))))))

(define-param nfreq 50)
(define refl (add-flux fcen df nfreq (make flux-region (center 0 0 (- (* 0.5 sz) tabs (* 0.5 tair))) (size a a 0))))
(define tran (add-flux fcen df nfreq (make flux-region (center 0 0 (+ (* -0.5 sz) tabs)) (size a a 0))))

(if (not empty?) (load-minus-flux "refl-flux" refl))
(run-sources+ (stop-when-fields-decayed 25 Ey (vector3 0 0 (- (* 0.5 sz) tabs (* 0.5 tair))) 1e-6))
(if empty? (save-flux "refl-flux" refl))

(display-fluxes refl tran)
